unit u_ld1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  IniFiles, StdCtrls, CheckLst, ExtCtrls;

type
  TForm1 = class(TForm)
    openDialog: TOpenDialog;
    chklstInfile: TCheckListBox;
    cmdOpen: TButton;
    cmdReplace: TButton;
    cmdFittings: TButton;
    cmdSave: TButton;
    cmdExit: TButton;
    lblFilename: TLabel;
    Panel: TPanel;
    lblBaseDir: TLabel;
    txtBaseDir: TEdit;
    memActivityLog: TMemo;
    lblSource: TLabel;
    lblBatch: TLabel;
    txtBatch: TEdit;
    btnBath: TButton;
    procedure FormCreate(Sender: TObject);
    procedure GetBaseDir;
    function ldExtractFromLine (ldline: String; Post: Integer): String;
    function ldLineType (ldline: String): Integer;
    function FloatToLDraw (inval: Double): String;
    function ldLineUpdate (ldline: String; ItemNr: Integer; NewVal: String): String;
    procedure cmdOpenClick(Sender: TObject);
    procedure cmdReplaceClick(Sender: TObject);
    procedure cmdFittingsClick(Sender: TObject);
    function isBox (fil: String): Boolean;
    procedure cmdSaveClick(Sender: TObject);
    procedure cmdExitClick(Sender: TObject);
    procedure memActivityLogChange(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btnBathClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
    fileList: TStringList;
    procedure DoIt(const extension: String);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;
  BaseDir: String;
  strPrefix: String; // tells the cmdReplaceClick what filename prefix to add
  // to files to box ('B\', 'B\T', or 'B\B')
  FileChanged: Boolean;
  replacedTotalCount: Integer;


implementation

{$R *.DFM}

procedure TForm1.FormCreate(Sender: TObject);
begin
  DecimalSeparator := '.';
  fileList:= TStringList.Create;
end;

procedure TForm1.DoIt(const extension: String);
var
  SR      : TSearchRec;
  CR      : TCursor;
  FN      : string;
  I       : integer;
begin
  if not DirectoryExists(txtBatch.Text) then Exit;
  if not DirectoryExists(txtBaseDir.Text) then Exit;
   cr := Screen.Cursor;
   Screen.Cursor := crHourglass;
   if FindFirst(txtBatch.Text + '\' + extension, faArchive, SR) = 0 then
   begin
     repeat
       chklstInfile.Clear;
       FN:= ExtractFileName(SR.Name);
       if fileList.Find(FN ,i) then continue;
       try
         FN:= txtBatch.Text + '\' + FN;
         if FileExists(FN) then
           chklstInfile.Items.LoadFromFile(FN);
       finally
         self.memActivityLog.Lines.Add('Loaded '+SR.Name);
       end;

       if FileExists(FN) then
       try
         cmdFittings.Click;
        finally
          if cmdSave.Enabled then begin
            chklstInfile.Items.SaveToFile(FN+'.modified.ldr');
            memActivityLog.Lines.Add('Saved ' + FN+'.modified.ldr');
            fileList.add(ExtractFileName(FN)+'.modified.ldr');
          end;
        end;
     until FindNext(SR) <> 0;
     FindClose(SR);
   end;
   Screen.Cursor := cr;
end;

procedure TForm1.GetBaseDir;
begin
  If txtBaseDir.GetTextLen > 0 then begin
    BaseDir:= txtBaseDir.Text;
    while (BaseDir[Length(BaseDir)]=' ')  or (BaseDir[Length(BaseDir)]='\') do
    begin
      BaseDir := copy(BaseDir, 1, Length(BaseDir)-1);
    end;
  end else BaseDir:= '.';
  BaseDir := BaseDir + '\';
end;


function TForm1.ldExtractFromLine (ldline: String; Post: Integer): String;
var
  s: String;
  t: array[1..15] of String;
  i, iT: Integer;
begin
  s := Trim(ldline);
  iT := 1;
  while s <> '' do
  begin
    s := Trim(s) + ' ';
    i := 1;
    while s[i] <> ' ' do Inc(i);
    t[iT] := Copy(s, 1, i-1);
    Inc(iT);
    s := Copy(s, i, Length(s));
  end;
  ldExtractFromLine := '';
  if Post < iT then ldExtractFromLine := t[Post];
end;


function TForm1.ldLineType (ldline: String): Integer;
var
  s: String;
begin
  ldLineType := -1;
  s := Trim(ldline);
  if Length(s)>0 then
  begin
    if s[1]='0' then ldLineType := 0;
    if s[1]='1' then ldLineType := 1;
    if s[1]='2' then ldLineType := 2;
    if s[1]='3' then ldLineType := 3;
    if s[1]='4' then ldLineType := 4;
    if s[1]='5' then ldLineType := 5;
  end;
end;


function TForm1.FloatToLDraw (inval: Double): String;
begin
  if (inval > -0.0001) and (inval < 0.0001) then inval := 0;
  FloatToLDraw := FloatToStr(Round(inval*10000)/10000);
end;


function TForm1.ldLineUpdate (ldline: String; ItemNr: Integer; NewVal: String): String;
var
  s: String;
  color1, fil: String;
  x1, y1, z1, a1, b1, c1, d1, e1, f1, g1, h1, i1: Double;
begin
  s := ldline;
  color1 := ldExtractFromLine(ldline, 2);
  x1 := StrToFloat(ldExtractFromLine(ldline, 3));
  y1 := StrToFloat(ldExtractFromLine(ldline, 4));
  z1 := StrToFloat(ldExtractFromLine(ldline, 5));
  a1 := StrToFloat(ldExtractFromLine(ldline, 6));
  b1 := StrToFloat(ldExtractFromLine(ldline, 7));
  c1 := StrToFloat(ldExtractFromLine(ldline, 8));
  d1 := StrToFloat(ldExtractFromLine(ldline, 9));
  e1 := StrToFloat(ldExtractFromLine(ldline, 10));
  f1 := StrToFloat(ldExtractFromLine(ldline, 11));
  g1 := StrToFloat(ldExtractFromLine(ldline, 12));
  h1 := StrToFloat(ldExtractFromLine(ldline, 13));
  i1 := StrToFloat(ldExtractFromLine(ldline, 14));
  fil := ldExtractFromLine(ldline, 15);
  case ItemNr of
    2: color1 := NewVal;
    3: x1 := StrToFloat(NewVal);
    4: y1 := StrToFloat(NewVal);
    5: z1 := StrToFloat(NewVal);
    6: a1 := StrToFloat(NewVal);
    7: b1 := StrToFloat(NewVal);
    8: c1 := StrToFloat(NewVal);
    9: d1 := StrToFloat(NewVal);
    10:e1 := StrToFloat(NewVal);
    11:f1 := StrToFloat(NewVal);
    12:g1 := StrToFloat(NewVal);
    13:h1 := StrToFloat(NewVal);
    14:i1 := StrToFloat(NewVal);
    15: fil := NewVal
  end;
  s := '1 ' + color1 + ' ';
  s := s + FloatToLDraw(x1)+ ' ';
  s := s + FloatToLDraw(y1)+ ' ';
  s := s + FloatToLDraw(z1)+ ' ';
  s := s + FloatToLDraw(a1)+ ' ';
  s := s + FloatToLDraw(b1)+ ' ';
  s := s + FloatToLDraw(c1)+ ' ';
  s := s + FloatToLDraw(d1)+ ' ';
  s := s + FloatToLDraw(e1)+ ' ';
  s := s + FloatToLDraw(f1)+ ' ';
  s := s + FloatToLDraw(g1)+ ' ';
  s := s + FloatToLDraw(h1)+ ' ';
  s := s + FloatToLDraw(i1)+ ' ' + fil;
  ldLineUpdate := s;
end;


procedure TForm1.cmdOpenClick(Sender: TObject);
var
  cursor: TCursor;
begin
  if OpenDialog.Execute then
  begin
   cursor := Screen.Cursor;
   Screen.Cursor := crHourglass;
   try
      self.memActivityLog.Lines.Add('Loading...');
      chklstInfile.Items.LoadFromFile(OpenDialog.FileName);
   finally
      Screen.Cursor := cursor;
      self.memActivityLog.Lines.Add('Loaded '+ExtractFileName(OpenDialog.FileName));
   end;
   FileChanged := False;
   cmdSave.Enabled := False;
   lblFilename.Caption := OpenDialog.FileName;
  end;

end;

procedure TForm1.cmdReplaceClick(Sender: TObject);
var
  i, replacedCount: Integer;
  ldline, fil: String;
begin
  replacedCount := 0;
  for i := 0 to (chklstInfile.Items.Count - 1) do begin
    ldline := chklstInfile.Items.Strings[i];
    if chklstInfile.Checked[i]=False then continue;
    if ldLineType(ldline) = 1 then begin
      fil := ldExtractFromLine(ldline, 15);
      //if isBox(fil) = False then fil := 'B\' + fil;
      if isBox(fil) = False then fil := strPrefix + fil;

      if FileExists(BaseDir + 'Parts\' + fil) then begin
        memActivityLog.Lines.Add('Found '+BaseDir + 'Parts\' + fil);
        ldline := ldLineUpdate(ldline, 15, fil);
        chklstInfile.Items.Strings[i] := ldline;
        Inc(replacedCount);
        Inc(replacedTotalCount);
      end else memActivityLog.Lines.Add('Missing!! '+BaseDir + 'Parts\' + fil);
    end;
  end;
  if replacedCount>0 then begin
    FileChanged := True;
    cmdSave.Enabled := True;
    chklstInfile.Items.Add('0 // Boxed ' + IntToStr(replacedCount)
     + ' parts (' + strPrefix + ').');

    memActivityLog.Lines.Add('Boxed ' + IntToStr(replacedCount)
     + ' parts (' + strPrefix + ').');
    // Moved v2010a chklstInfile.Items.Add('0');
  end;
end;

procedure TForm1.cmdFittingsClick(Sender: TObject);
var
  i, ii, iii, frg: Integer;
  s, ldline, fil: String;
  x1, y1, z1, a1, c1, g1, i1, x2, y2, z2, x3, y3, z3: Double;
  StrListInfil, StrLstCover: TStringList;
  SkipThis: Boolean;
  cursor: TCursor;
begin
  if not DirectoryExists(txtBaseDir.Text) then Exit;
  GetBaseDir;
  cursor := Screen.Cursor;
  Screen.Cursor := crHourglass;
  try
  StrListInfil := TStringList.Create;
  StrLstCover := TStringList.Create;
  StrLstCover.Clear;
  replacedTotalCount := 0;
  //
  // cmdFittingsClick STEP 1: Compile a list of locations (20x20LDU)
  // Tops and Bottoms that are hiding (=covering) details from next part
  // Save the list in StrListCover
  for i := 0 to (chklstInfile.Items.Count - 1) do begin
    memActivityLog.Lines.Add('Check Line '+IntToStr(i));
    ldline := chklstInfile.Items.Strings[i];
    if ldLineType(ldline) <> 1 then continue;
    frg := StrToInt(ldExtractFromLine(ldline, 2));
    if (frg>31) and (frg<48) then continue;
    if ldExtractFromLine(ldline, 7) <> '0' then continue;
    if ldExtractFromLine(ldline, 9) <> '0' then continue;
    if ldExtractFromLine(ldline, 10) <> '1' then continue;
    if ldExtractFromLine(ldline, 11) <> '0' then continue;
    if ldExtractFromLine(ldline, 13) <> '0' then continue;

    fil := ldExtractFromLine(ldline, 15);
    if Length(fil)<5 then continue; // just to be foolproof...

    // 2010-03-20 also check if already boxed parts cover positions
    // by removing B\, B\t, or B\B from examined file reference
    if UpperCase(Copy(fil,1,3))='B\T' then
    fil := Copy(fil, 4, Length(fil)-3);
    if UpperCase(Copy(s,1,3))='B\B' then
    fil := Copy(fil, 4, Length(fil)-3);
    if UpperCase(Copy(fil,1,2))='B\' then
    fil := Copy(fil, 3, Length(fil)-2);
    if Length(fil)<5 then continue;

    fil := Copy (fil, 1, Length(fil)-4);
    fil := BaseDir+'Parts\B\' + fil + '.nfo';
    memActivityLog.Lines.Add('Searching for '+fil);
    if FileExists(fil) = False then continue;

    x1 := StrToFloat(ldExtractFromLine(ldline, 3));
    y1 := StrToFloat(ldExtractFromLine(ldline, 4));
    z1 := StrToFloat(ldExtractFromLine(ldline, 5));
    a1 := StrToFloat(ldExtractFromLine(ldline, 6));
    c1 := StrToFloat(ldExtractFromLine(ldline, 8));
    g1 := StrToFloat(ldExtractFromLine(ldline, 12));
    i1 := StrToFloat(ldExtractFromLine(ldline, 14));

    StrListInfil.Clear;
    StrListInfil.LoadFromFile(fil);
    for ii := 0 to (StrListInfil.Count - 1) do begin
        ldline := StrListInfil.Strings[ii];
        s := ldExtractFromLine(ldline, 1);
        x2 := StrToFloat(ldExtractFromLine(ldline, 2));
        y2 := StrToFloat(ldExtractFromLine(ldline, 3));
        z2 := StrToFloat(ldExtractFromLine(ldline, 4));
        if (s='Top') or (s='Stud') then s := 'T ';
        if s='Bottom' then s := 'B ';
        x3 := x1 + x2*a1 + z2*c1;
        y3 := y1 + y2;
        z3 := z1 + x2*g1 + z2*i1;
        s := s + FloatToLDraw(x3) + ' ';
        s := s + FloatToLDraw(y3) + ' ';
        s := s + FloatToLDraw(z3);
        StrLstCover.Add(s);
    end;
  end;
  // END of cmdFittingsClick STEP 1
  // The model file in chklstInfile has been scanned for locations
  //  meeting all the given criterias
  // For example:
  // This following line meets the criteria:
  // 1 15  240 -48 160  1 0 0  0 1 0  0 0 1  3005.dat
  // The two lines from the file Parts\B\3005.nfo is used:
  // Stud 0 0 0
  // Bottom 0 24 0
  // The output stored in StrListCover is the following two lines:
  // T 240 -48 160
  // B 240 -24 160
  //

  //
  // cmdFittingsClick STEP 2: Replace all parts that can be fully boxed
  //  Checkbox all parts that has all stud and bottom locations covered
  //  according to StrListCover
  // (Checkboxing is not needed and slows down the procedure
  //  It is a remnant from the time LDBoxer was partially manual
  //  I keep it just in case a future version provides more manual options)
  for i := 0 to (chklstInfile.Items.Count - 1) do begin
    ldline := chklstInfile.Items.Strings[i];
    if ldLineType(ldline) <> 1 then continue;
    frg := StrToInt(ldExtractFromLine(ldline, 2));
    if (frg>31) and (frg<48) then continue;
    if ldExtractFromLine(ldline, 7) <> '0' then continue;
    if ldExtractFromLine(ldline, 9) <> '0' then continue;
    if ldExtractFromLine(ldline, 10) <> '1' then continue;
    if ldExtractFromLine(ldline, 11) <> '0' then continue;
    if ldExtractFromLine(ldline, 13) <> '0' then continue;
    fil := ldExtractFromLine(ldline, 15);
    if Length(fil)<5 then continue;
    fil := Copy (fil, 1, Length(fil)-4);

    fil := BaseDir+'Parts\B\' + fil + '.dat';
    memActivityLog.Lines.Add('Searching for '+fil);
    if FileExists(fil) = False then continue;

    fil := Copy (fil, 1, Length(fil)-4);
    fil := fil + '.nfo';
    if FileExists(fil) = False then continue;
    x1 := StrToFloat(ldExtractFromLine(ldline, 3));
    y1 := StrToFloat(ldExtractFromLine(ldline, 4));
    z1 := StrToFloat(ldExtractFromLine(ldline, 5));
    a1 := StrToFloat(ldExtractFromLine(ldline, 6));
    c1 := StrToFloat(ldExtractFromLine(ldline, 8));
    g1 := StrToFloat(ldExtractFromLine(ldline, 12));
    i1 := StrToFloat(ldExtractFromLine(ldline, 14));
    StrListInfil.Clear;
    StrListInfil.LoadFromFile(fil);
    for ii := 0 to (StrListInfil.Count - 1) do begin
      ldline := StrListInfil.Strings[ii];
      s := ldExtractFromLine(ldline, 1);
      x2 := StrToFloat(ldExtractFromLine(ldline, 2));
      y2 := StrToFloat(ldExtractFromLine(ldline, 3));
      z2 := StrToFloat(ldExtractFromLine(ldline, 4));
      if s='Top' then continue;
      if s='Stud' then s := 'B '; //Scan for matching B
      if s='Bottom' then s := 'T '; //Scan for matching T
      x3 := x1 + x2*a1 + z2*c1;
      y3 := y1 + y2;
      z3 := z1 + x2*g1 + z2*i1;
      s := s + FloatToLDraw(x3) + ' ';
      s := s + FloatToLDraw(y3) + ' ';
      s := s + FloatToLDraw(z3);
      SkipThis := True;
      for iii := 0 to (StrLstCover.Count - 1) do begin
        if s=StrLstCover.Strings[iii] then SkipThis := False;
      end;
      if SkipThis then break;
    end;
    if SkipThis then continue;
    chklstInfile.Checked[i] := True;
  end;
  strPrefix := 'B\';
  cmdReplace.Click;
  // END of cmdFittingsClick STEP 2
  //

  //
  // cmdFittingsClick STEP 3: Replace all parts that studs can be removed from
  // (STEPs 3 & 4 should be easily baked into STEP 2...)
  for i := 0 to (chklstInfile.Items.Count - 1) do begin
    chklstInfile.Checked[i] := False;
    ldline := chklstInfile.Items.Strings[i];
    if ldLineType(ldline) <> 1 then continue;
    frg := StrToInt(ldExtractFromLine(ldline, 2));
    if (frg>31) and (frg<48) then continue;
    if ldExtractFromLine(ldline, 7) <> '0' then continue;
    if ldExtractFromLine(ldline, 9) <> '0' then continue;
    if ldExtractFromLine(ldline, 10) <> '1' then continue;
    if ldExtractFromLine(ldline, 11) <> '0' then continue;
    if ldExtractFromLine(ldline, 13) <> '0' then continue;
    fil := ldExtractFromLine(ldline, 15);
    if Length(fil)<5 then continue;
    fil := Copy (fil, 1, Length(fil)-4);

    fil := BaseDir+'Parts\B\' + fil + '.dat';
    memActivityLog.Lines.Add('Searching for '+fil);
    if FileExists(fil) = False then continue;

    fil := Copy (fil, 1, Length(fil)-4);
    fil := fil + '.nfo';
    if FileExists(fil) = False then continue;
    x1 := StrToFloat(ldExtractFromLine(ldline, 3));
    y1 := StrToFloat(ldExtractFromLine(ldline, 4));
    z1 := StrToFloat(ldExtractFromLine(ldline, 5));
    a1 := StrToFloat(ldExtractFromLine(ldline, 6));
    c1 := StrToFloat(ldExtractFromLine(ldline, 8));
    g1 := StrToFloat(ldExtractFromLine(ldline, 12));
    i1 := StrToFloat(ldExtractFromLine(ldline, 14));
    StrListInfil.Clear;
    StrListInfil.LoadFromFile(fil);
    SkipThis := False;
    for ii := 0 to (StrListInfil.Count - 1) do begin
      ldline := StrListInfil.Strings[ii];
      s := ldExtractFromLine(ldline, 1);
      x2 := StrToFloat(ldExtractFromLine(ldline, 2));
      y2 := StrToFloat(ldExtractFromLine(ldline, 3));
      z2 := StrToFloat(ldExtractFromLine(ldline, 4));
      if s='Top' then continue;
      //if s='Stud' then continue;
      if s='Stud' then s := 'B '; //Scan for matching B
      if s='Bottom' then continue;
      //if s='Bottom' then s := 'T '; //Scan for matching T
      x3 := x1 + x2*a1 + z2*c1;
      y3 := y1 + y2;
      z3 := z1 + x2*g1 + z2*i1;
      s := s + FloatToLDraw(x3) + ' ';
      s := s + FloatToLDraw(y3) + ' ';
      s := s + FloatToLDraw(z3);
      SkipThis := True;
      for iii := 0 to (StrLstCover.Count - 1) do begin
        if s=StrLstCover.Strings[iii] then SkipThis := False;
      end;
      if SkipThis then break;
    end;
    if SkipThis then continue;
    chklstInfile.Checked[i] := True;
  end;
  strPrefix := 'B\T';
  cmdReplace.Click;
  // END of cmdFittingsClick STEP 3
  //

  //
  // cmdFittingsClick STEP 4: Replace all parts that bottom details
  //  can be removed from
  // (STEPs 3 & 4 should be easily baked into STEP 2...)
  for i := 0 to (chklstInfile.Items.Count - 1) do begin
    chklstInfile.Checked[i] := False;
    ldline := chklstInfile.Items.Strings[i];
    if ldLineType(ldline) <> 1 then continue;
    frg := StrToInt(ldExtractFromLine(ldline, 2));
    if (frg>31) and (frg<48) then continue;
    if ldExtractFromLine(ldline, 7) <> '0' then continue;
    if ldExtractFromLine(ldline, 9) <> '0' then continue;
    if ldExtractFromLine(ldline, 10) <> '1' then continue;
    if ldExtractFromLine(ldline, 11) <> '0' then continue;
    if ldExtractFromLine(ldline, 13) <> '0' then continue;
    fil := ldExtractFromLine(ldline, 15);
    if Length(fil)<5 then continue;
    fil := Copy (fil, 1, Length(fil)-4);

    fil := BaseDir+'Parts\B\' + fil + '.dat';
    memActivityLog.Lines.Add('Searching for '+fil);
    if FileExists(fil) = False then continue;

    fil := Copy (fil, 1, Length(fil)-4);
    fil := fil + '.nfo';
    if FileExists(fil) = False then continue;
    x1 := StrToFloat(ldExtractFromLine(ldline, 3));
    y1 := StrToFloat(ldExtractFromLine(ldline, 4));
    z1 := StrToFloat(ldExtractFromLine(ldline, 5));
    a1 := StrToFloat(ldExtractFromLine(ldline, 6));
    c1 := StrToFloat(ldExtractFromLine(ldline, 8));
    g1 := StrToFloat(ldExtractFromLine(ldline, 12));
    i1 := StrToFloat(ldExtractFromLine(ldline, 14));
    StrListInfil.Clear;
    StrListInfil.LoadFromFile(fil);
    SkipThis := False;
    for ii := 0 to (StrListInfil.Count - 1) do begin
      ldline := StrListInfil.Strings[ii];
      s := ldExtractFromLine(ldline, 1);
      x2 := StrToFloat(ldExtractFromLine(ldline, 2));
      y2 := StrToFloat(ldExtractFromLine(ldline, 3));
      z2 := StrToFloat(ldExtractFromLine(ldline, 4));
      if s='Top' then continue;
      if s='Stud' then continue;
      if s='Stud' then s := 'B '; //Scan for matching B
      //if s='Bottom' then continue;
      if s='Bottom' then s := 'T '; //Scan for matching T
      x3 := x1 + x2*a1 + z2*c1;
      y3 := y1 + y2;
      z3 := z1 + x2*g1 + z2*i1;
      s := s + FloatToLDraw(x3) + ' ';
      s := s + FloatToLDraw(y3) + ' ';
      s := s + FloatToLDraw(z3);
      SkipThis := True;
      for iii := 0 to (StrLstCover.Count - 1) do begin
        if s=StrLstCover.Strings[iii] then SkipThis := False;
      end;
      if SkipThis then break;
    end;
    if SkipThis then continue;
    chklstInfile.Checked[i] := True;
  end;
  strPrefix := 'B\B';
  cmdReplace.Click;
  // END of cmdFittingsClick STEP 4
  //
  for i := 0 to (chklstInfile.Items.Count - 1) do begin
    chklstInfile.Checked[i] := False;
  end;
  if replacedTotalCount>0 then begin
    // 0 !LDOXER LEVEL info should also be automatically updated!
    chklstInfile.Items.Add('0 // Boxed Total ' + IntToStr(replacedTotalCount)
     + ' parts by LDBoxer v2010a.');
    chklstInfile.Items.Add('0');
  end;

  StrLstCover.Free;
  StrListInfil.Free;
  finally
    Screen.Cursor := cursor;
    memActivityLog.Lines.Add('Finished task.');
  end;
end;


function TForm1.isBox (fil: String): Boolean;
begin
  if UpperCase(Copy(fil,1,2)) = 'B\' then begin
    isBox := True
  end
  else begin
    isBox := False;
  end;
end;

procedure TForm1.cmdSaveClick(Sender: TObject);
begin
  chklstInfile.Items.SaveToFile(OpenDialog.FileName);
  FileChanged := False;
  cmdSave.Enabled := False;
  memActivityLog.Lines.Add('Saved ' + OpenDialog.FileName+'.modified.ldr');
  fileList.Add(ExtractFileName(OpenDialog.FileName)+'.modified.ldr');
end;

procedure TForm1.cmdExitClick(Sender: TObject);
begin
  Close;
end;

procedure TForm1.memActivityLogChange(Sender: TObject);
begin
{@see https://stackoverflow.com/questions/4322846/autoscrolling-memo-in-delphi
 @note If your memo is read-only to users and is updated automatically by the application,
       you can put a call to the above procedure in its OnChange event-handler,
       so that whenever the text inside the memo is changed,
       it is automatically scrolled down to the last line.
}
   SendMessage(memActivityLog.Handle, EM_LINESCROLL, 0,memActivityLog.Lines.Count);
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
   memActivityLog.Lines.saveToFile(ExtractFilePath(Application.ExeName)+'\ldboxer_stdout.txt');
end;

procedure TForm1.btnBathClick(Sender: TObject);
begin
  DoIt('*.mpd'); cmdSave.Enabled:= false;
  DoIt('*.ldr'); cmdSave.Enabled:= false;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  fileList.Free;
end;

end.
